<Serializable()> _
Public NotInheritable Class GenericControlList(Of Element)
    Implements IList, ICollection, IEnumerable

    Dim List As New List(Of Element)

    Event OnAdd(ByVal item As Element)
    Event OnClear()
    Event OnRemove(ByVal item As Element)
    Event OnInsert(ByVal index As Integer, ByVal item As Element)
    Event OnRemoveAt(ByVal index As Integer)
    Event OnSetItem(ByVal index As Integer, ByVal item As Element)
    Event OnGetItem(ByVal index As Integer)


    Public Sub CopyTo(ByVal array As System.Array, ByVal index As Integer) Implements System.Collections.ICollection.CopyTo
        List.CopyTo(array, index)
    End Sub

    Public ReadOnly Property Count() As Integer Implements System.Collections.ICollection.Count
        Get
            Return List.Count
        End Get
    End Property

    Public ReadOnly Property IsSynchronized() As Boolean Implements System.Collections.ICollection.IsSynchronized
        Get
            Return DirectCast(List, ICollection).IsSynchronized
        End Get
    End Property

    Public ReadOnly Property SyncRoot() As Object Implements System.Collections.ICollection.SyncRoot
        Get
            Return DirectCast(List, ICollection).SyncRoot
        End Get
    End Property

    Public Function GetEnumerator() As System.Collections.IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
        Return List.GetEnumerator
    End Function
    Public Function Add(ByVal value As Element) As Integer
        Add = DirectCast(List, IList).Add(value)
        RaiseEvent OnAdd(value)
    End Function

    Private Function Add(ByVal value As Object) As Integer Implements System.Collections.IList.Add
        Add = DirectCast(List, IList).Add(DirectCast(value, Element))
        RaiseEvent OnAdd(value)
    End Function

    Private Sub AddRange(ByVal items As Element())
        Me.List.AddRange(items)
        For Each e As Element In items
            RaiseEvent OnAdd(e)
        Next
    End Sub

    Private Sub AddRange(ByVal items As GenericControlList(Of Element))
        Me.List.AddRange(items)
        For Each e As Element In items
            RaiseEvent OnAdd(e)
        Next
    End Sub

    Public Sub Clear() Implements System.Collections.IList.Clear
        List.Clear()
        RaiseEvent OnClear()
    End Sub

    Public Function Contains(ByVal value As Element) As Boolean
        Return List.Contains(value)
    End Function

    Private Function Contains(ByVal value As Object) As Boolean Implements System.Collections.IList.Contains
        Return List.Contains(DirectCast(value, Element))
    End Function

    Public Function IndexOf(ByVal value As Element) As Integer
        Return List.IndexOf(value)
    End Function

    Private Function IndexOf(ByVal value As Object) As Integer Implements System.Collections.IList.IndexOf
        Return List.IndexOf(DirectCast(value, Element))
    End Function

    Public Sub Insert(ByVal index As Integer, ByVal value As Element)
        List.Insert(index, value)
        RaiseEvent OnInsert(index, value)
    End Sub
    Private Sub Insert(ByVal index As Integer, ByVal value As Object) Implements System.Collections.IList.Insert
        List.Insert(index, value)
        RaiseEvent OnInsert(index, value)
    End Sub


    Public ReadOnly Property IsFixedSize() As Boolean Implements System.Collections.IList.IsFixedSize
        Get
            Return DirectCast(List, IList).IsFixedSize
        End Get
    End Property

    Public ReadOnly Property IsReadOnly() As Boolean Implements System.Collections.IList.IsReadOnly
        Get
            Return DirectCast(List, IList).IsReadOnly
        End Get
    End Property
    Default Public Property Item(ByVal index As Integer) As Element
        Get
            RaiseEvent OnGetItem(index)
            Return List(index)
        End Get
        Set(ByVal value As Element)
            List(index) = value
            RaiseEvent OnSetItem(index, value)
        End Set
    End Property

    Private Property pItem(ByVal index As Integer) As Object Implements System.Collections.IList.Item
        Get
            RaiseEvent OnGetItem(index)
            Return List(index)
        End Get
        Set(ByVal value As Object)
            List(index) = value
            RaiseEvent OnSetItem(index, value)
        End Set
    End Property


    Public Sub Remove(ByVal value As Element)
        List.Remove(value)
        RaiseEvent OnRemove(value)
    End Sub
    Private Sub Remove(ByVal value As Object) Implements System.Collections.IList.Remove
        List.Remove(DirectCast(value, Element))
        RaiseEvent OnRemove(value)
    End Sub

    Public Sub RemoveAt(ByVal index As Integer) Implements System.Collections.IList.RemoveAt
        List.RemoveAt(index)
        RaiseEvent OnRemoveAt(index)
    End Sub
End Class